@file:JvmName("Compilations")

package ru.landgrafhomyak.kotlin.kmp_gradle_build_helper

import org.jetbrains.kotlin.gradle.dsl.KotlinCommonCompilerOptions
import org.jetbrains.kotlin.gradle.dsl.KotlinMultiplatformExtension
import org.jetbrains.kotlin.gradle.plugin.KotlinCompilation
import org.jetbrains.kotlin.gradle.dsl.KotlinVersion
import org.jetbrains.kotlin.gradle.plugin.KotlinTarget

public fun KotlinMultiplatformExtension.configureAllCompilationsOnAllTargets(action: KotlinCompilation<*>.() -> Unit) {
	this.targets.configureEach { t ->
		t.compilations.configureEach(action)
	}
}

@Deprecated(
	message = "New name avoids confusions when function called on wrong scope",
	replaceWith = ReplaceWith(
		"this.configureAllCompilationsOnAllTargets",
		"ru.landgrafhomyak.kotlin.kmp_gradle_build_helper.configureAllCompilationsOnAllTargets"
	)
)
public fun KotlinMultiplatformExtension.configureAllCompilations(action: KotlinCompilation<*>.() -> Unit): Unit =
	this.configureAllCompilationsOnAllTargets(action)

public fun KotlinMultiplatformExtension.configureAllCompilersOptionsOnAllTargets(action: KotlinCommonCompilerOptions.() -> Unit) {
	this.targets.configureEach { t ->
		t.configureAllCompilersOptionsOnAllCompilations(action)
	}
}

@Deprecated(
	message = "New name avoids confusions when function called on wrong scope",
	replaceWith = ReplaceWith(
		"this.configureAllCompilersOptionsOnAllTargets",
		"ru.landgrafhomyak.kotlin.kmp_gradle_build_helper.configureAllCompilersOptionsOnAllTargets"
	)
)
public fun KotlinMultiplatformExtension.configureAllCompilersOptions(action: KotlinCommonCompilerOptions.() -> Unit): Unit =
	this.configureAllCompilersOptionsOnAllTargets(action)

public fun KotlinTarget.configureAllCompilersOptionsOnAllCompilations(action: KotlinCommonCompilerOptions.() -> Unit) {
	this.compilations.configureEach { c ->
		c.compileTaskProvider.configure { t ->
			t.compilerOptions(action)
		}
	}
}

public fun KotlinMultiplatformExtension.noWarnExpectActual(): Unit =
	this.configureAllCompilersOptionsOnAllTargets { this@configureAllCompilersOptionsOnAllTargets.freeCompilerArgs.add("-Xexpect-actual-classes") }

public fun KotlinMultiplatformExtension.warningsAsErrors(): Unit =
	this.configureAllCompilersOptionsOnAllTargets { this@configureAllCompilersOptionsOnAllTargets.allWarningsAsErrors.set(true) }

public fun KotlinMultiplatformExtension.optIn(classQualname: String): Unit =
	this.configureAllCompilersOptionsOnAllTargets { this@configureAllCompilersOptionsOnAllTargets.optIn.add(classQualname) }

public fun KotlinMultiplatformExtension.optInContracts(): Unit =
	this.optIn("kotlin.contracts.ExperimentalContracts")

public fun KotlinMultiplatformExtension.optInUnsignedArrayTypes(): Unit =
	this.optIn("kotlin.ExperimentalUnsignedTypes")

public fun KotlinMultiplatformExtension.setCompatibilityWithKotlin(version: KotlinVersion): Unit =
	this.configureAllCompilersOptionsOnAllTargets {
		this@configureAllCompilersOptionsOnAllTargets.apiVersion.set(version)
		this@configureAllCompilersOptionsOnAllTargets.languageVersion.set(version)
	}

public fun KotlinTarget.suppressWarningsForAllCompilations(vararg names: String): Unit =
	this.configureAllCompilersOptionsOnAllCompilations {
		for (warn in names) {
			this@configureAllCompilersOptionsOnAllCompilations.freeCompilerArgs.add("-Xsuppress-warning=${warn}")
		}
	}

public fun KotlinMultiplatformExtension.suppressWarningsForForAllTargets(vararg names: String) {
	this.targets.configureEach { t ->
		t.suppressWarningsForAllCompilations(*names)
	}
}
